/*
 * Copyright (C) 2008-2014 Codership Oy <info@codership.com>
 *
 * $Id$
 */

#ifndef _gcs_gcomm_h_
#define _gcs_gcomm_h_

#include "gcs_backend.hpp"

extern GCS_BACKEND_REGISTER_FN(gcs_gcomm_register);

extern GCS_BACKEND_CREATE_FN(gcs_gcomm_create);

#endif /* _gcs_vs_h_ */
