/* Copyright (C) 2013 Codership Oy <info@codership.com>
 *
 * $Id$
 */
#ifndef __gu_vec_test__
#define __gu_vec_test__

#include <check.h>

extern Suite *gu_vec_suite(void);

#endif /* __gu_vec_test__ */
