// Copyright (C) 2013 Codership Oy <info@codership.com>

/**
 * @file Location of some "standard" types definitions
 *
 * $Id$
 */

#ifndef _GU_TYPES_HPP_
#define _GU_TYPES_HPP_

#include "gu_types.h"

namespace gu {

typedef gu_byte_t byte_t;

}

#endif /* _GU_TYPES_HPP_ */
