/*
 * Copyright (C) 2008 Codership Oy <info@codership.com>
 *
 * $Id$
 */
/*
 *  Definition of Spread GC backend
 */

#ifndef _gcs_spread_h_
#define _gcs_spread_h_

#include "gcs_backend.h"

extern GCS_BACKEND_CREATE_FN  (gcs_spread_create);

#endif /* _gcs_spread_h_ */
